"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from test.integration import BaseCliTestCase

import cfnlint.core


class TestQuickStartTemplates(BaseCliTestCase):
    """Test QuickStart Templates Parsing"""

    scenarios = [
        {
            "filename": "test/fixtures/templates/good/generic.yaml",
            "results": [],
            "exit_code": 0,
        },
        {
            "filename": "test/fixtures/templates/good/minimal.yaml",
            "results": [],
            "exit_code": 0,
        },
        {
            "filename": "test/fixtures/templates/good/transform.yaml",
            "results": [],
            "exit_code": 0,
        },
        {
            "filename": "test/fixtures/templates/issues/sam_w_conditions.yaml",
            "results": [],
            "exit_code": 0,
        },
        {
            "filename": "test/fixtures/templates/bad/transform_serverless_template.yaml",
            "results": [
                {
                    "Filename": "test/fixtures/templates/bad/transform_serverless_template.yaml",
                    "Location": {
                        "Start": {"ColumnNumber": 1, "LineNumber": 1},
                        "End": {"ColumnNumber": 1, "LineNumber": 1},
                        "Path": None,
                    },
                    "Rule": {
                        "Id": "E0001",
                        "Description": "Errors found when performing transformation on the template",
                        "Source": "https://github.com/aws-cloudformation/cfn-python-lint",
                        "ShortDescription": "Error found when transforming the template",
                    },
                    "Level": "Error",
                    "Message": "Error transforming template: Resource with id [AppName] is invalid. Resource is missing the required [Location] property.",
                },
                {
                    "Filename": "test/fixtures/templates/bad/transform_serverless_template.yaml",
                    "Location": {
                        "Start": {"ColumnNumber": 1, "LineNumber": 1},
                        "End": {"ColumnNumber": 1, "LineNumber": 1},
                        "Path": None,
                    },
                    "Rule": {
                        "Id": "E0001",
                        "Description": "Errors found when performing transformation on the template",
                        "Source": "https://github.com/aws-cloudformation/cfn-python-lint",
                        "ShortDescription": "Error found when transforming the template",
                    },
                    "Level": "Error",
                    "Message": "Error transforming template: Resource with id [ExampleLayer] is invalid. Missing required property 'ContentUri'.",
                },
                {
                    "Filename": "test/fixtures/templates/bad/transform_serverless_template.yaml",
                    "Location": {
                        "Start": {"ColumnNumber": 1, "LineNumber": 1},
                        "End": {"ColumnNumber": 1, "LineNumber": 1},
                        "Path": None,
                    },
                    "Rule": {
                        "Id": "E0001",
                        "Description": "Errors found when performing transformation on the template",
                        "Source": "https://github.com/aws-cloudformation/cfn-python-lint",
                        "ShortDescription": "Error found when transforming the template",
                    },
                    "Level": "Error",
                    "Message": "Error transforming template: Resource with id [myFunctionMyTimer] is invalid. Missing required property 'Schedule'.",
                },
            ],
            "exit_code": 2,
        },
        {
            "filename": "test/fixtures/templates/good/conditions.yaml",
            "results": [],
            "exit_code": 0,
        },
        {
            "filename": "test/fixtures/templates/good/resources_codepipeline.yaml",
            "results": [],
            "exit_code": 0,
        },
        {
            "filename": "test/fixtures/templates/good/resources_cognito_userpool_tag_is_string_map.yaml",
            "results": [],
            "exit_code": 0,
        },
        {
            "filename": "test/fixtures/templates/bad/resources_cognito_userpool_tag_is_list.yaml",
            "results": [
                {
                    "Filename": "test/fixtures/templates/bad/resources_cognito_userpool_tag_is_list.yaml",
                    "Level": "Error",
                    "Location": {
                        "Start": {"ColumnNumber": 7, "LineNumber": 16},
                        "End": {"ColumnNumber": 19, "LineNumber": 16},
                        "Path": [
                            "Resources",
                            "MyCognitoUserPool",
                            "Properties",
                            "UserPoolTags",
                        ],
                    },
                    "Message": "Map must be an object of key-value pairs",
                    "Rule": {
                        "Description": "Checks resource property values with Primitive Types for values that match those types.",
                        "Id": "E3012",
                        "ShortDescription": "Check resource properties values",
                        "Source": "https://github.com/aws-cloudformation/cfn-python-lint/blob/main/docs/cfn-resource-specification.md#valueprimitivetype",
                    },
                }
            ],
            "exit_code": 2,
        },
        {
            "filename": "test/fixtures/templates/good/transform_serverless_api.yaml",
            "results": [],
            "exit_code": 0,
        },
        {
            "filename": "test/fixtures/templates/good/transform_serverless_function.yaml",
            "results": [],
            "exit_code": 0,
        },
        {
            "filename": "test/fixtures/templates/good/transform_serverless_globals.yaml",
            "results": [
                {
                    "Filename": "test/fixtures/templates/good/transform_serverless_globals.yaml",
                    "Level": "Error",
                    "Location": {
                        "End": {"ColumnNumber": 13, "LineNumber": 10},
                        "Path": ["Resources", "myFunction", "Properties", "Runtime"],
                        "Start": {"ColumnNumber": 3, "LineNumber": 10},
                    },
                    "Message": "Deprecated runtime (nodejs6.10) specified. Updating disabled since 2019-08-12. Please consider updating to nodejs16.x",
                    "Rule": {
                        "Description": "Check if an EOL Lambda Runtime is specified and give an error if used. ",
                        "Id": "E2531",
                        "ShortDescription": "Check if EOL Lambda Function Runtimes are used",
                        "Source": "https://docs.aws.amazon.com/lambda/latest/dg/runtime-support-policy.html",
                    },
                }
            ],
            "exit_code": 2,
        },
        {
            "filename": "test/fixtures/templates/good/transform/list_transform.yaml",
            "results": [],
            "exit_code": 0,
        },
        {
            "filename": "test/fixtures/templates/good/transform/list_transform_many.yaml",
            "results": [],
            "exit_code": 0,
        },
        {
            "filename": "test/fixtures/templates/good/transform/list_transform_not_sam.yaml",
            "results": [],
            "exit_code": 0,
        },
    ]

    def test_templates(self):
        """Test Successful JSON Parsing"""
        self.run_scenarios()

    def test_module_integration(self):
        """Test same templates using integration approach"""
        rules = cfnlint.core.get_rules([], [], ["E", "W"], {}, False)
        self.run_module_integration_scenarios(rules)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.resources.properties.ValueRefGetAtt import (
    ValueRefGetAtt,  # pylint: disable=E0401
)


class TestValueRefGetAtt(BaseRuleTestCase):
    """Test Password Property Configuration"""

    def setUp(self):
        """Setup"""
        super(TestValueRefGetAtt, self).setUp()
        self.collection.register(ValueRefGetAtt())
        self.success_templates = [
            "test/fixtures/templates/good/resources/properties/value.yaml",
            "test/fixtures/templates/good/properties_ec2_vpc.yaml",
        ]

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_negative(self):
        """Test failure"""
        self.helper_file_negative("test/fixtures/templates/bad/generic.yaml", 2)

    def test_file_negative_value(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/properties/value.yaml", 8
        )

    def test_file_negative_vpc_id_value(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/properties_vpcid.yaml", 1
        )

    def test_file_negative_az(self):
        """Test failure"""
        self.helper_file_negative("test/fixtures/templates/bad/parameters/az.yaml", 3)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
import json
from test.unit.rules import BaseRuleTestCase

import cfnlint.helpers
from cfnlint.rules.resources.properties.Required import (
    Required,  # pylint: disable=E0401
)


class TestResourceConfiguration(BaseRuleTestCase):
    """Test Resource Properties"""

    def setUp(self):
        """Setup"""
        super(TestResourceConfiguration, self).setUp()
        self.collection.register(Required())
        self.success_templates = [
            "test/fixtures/templates/good/resources/properties/required.yaml",
        ]

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_negative(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/properties_required.yaml", 12
        )

    def test_file_negative_generic(self):
        """Generic Test failure"""
        self.helper_file_negative("test/fixtures/templates/bad/generic.yaml", 8)


class TestSpecifiedCustomResourceRequiredProperties(TestResourceConfiguration):
    """Test Custom Resource Required Properties when override spec is provided"""

    def setUp(self):
        """Setup"""
        super(TestSpecifiedCustomResourceRequiredProperties, self).setUp()
        # Add a Spec override that specifies the Custom::SpecifiedCustomResource type
        with open("test/fixtures/templates/override_spec/custom.json") as fp:
            custom_spec = json.load(fp)
        cfnlint.helpers.set_specs(custom_spec)
        # Reset Spec override after test
        self.addCleanup(cfnlint.helpers.initialize_specs)

    # ... all TestResourceConfiguration test cases are re-run with override spec ...

    def test_file_negative(self):
        """Test failure"""
        # Additional Custom::SpecifiedCustomResource failure detected with custom spec
        self.helper_file_negative(
            "test/fixtures/templates/bad/properties_required.yaml", 13
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from test.testlib.testcase import BaseTestCase

from cfnlint.decode.node import (  # pylint: disable=E0401
    TemplateAttributeError,
    intrinsic_node,
)


class TestNodeIntrinsic(BaseTestCase):
    def test_success_init(self):
        template = intrinsic_node({"Fn::Sub": []}, (0, 1), (2, 3))

        self.assertRaises(TemplateAttributeError, template.is_valid)



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from test.testlib.testcase import BaseTestCase

from cfnlint.data import CloudSpecs
from cfnlint.helpers import RESOURCE_SPECS, load_resource


class TestPatchedSpecs(BaseTestCase):
    """Test Patched spec files"""

    found_value_types = []

    def setUp(self):
        """SetUp template object"""

        self.spec = RESOURCE_SPECS["us-east-1"]

    def _test_property_type_values(self, values, r_name, p_name, r_type):
        p_value_type = values.get("Value", {}).get("ValueType")
        if p_value_type:
            self.found_value_types.append(p_value_type)
            self.assertIn(
                p_value_type,
                self.spec.get("ValueTypes"),
                "%s: %s, Property: %s. %s not found"
                % (r_type, r_name, p_name, p_value_type),
            )
            # List Value if a singular value is set and the type is List
            if values.get("Type") == "List":
                p_list_value_type = values.get("Value", {}).get("ListValueType")
                if p_list_value_type:
                    self.found_value_types.append(p_list_value_type)
                    self.assertIn(
                        p_list_value_type,
                        self.spec.get("ValueTypes"),
                        "%s: %s, Property: %s. %s not found"
                        % (r_type, r_name, p_name, p_value_type),
                    )

    def test_resource_type_values(self):
        """Test Resource Type Value"""
        r_type = "ResourceTypes"
        for r_name, r_values in self.spec.get("ResourceTypes").items():
            for p_name, p_values in r_values.get("Properties").items():
                self._test_property_type_values(p_values, r_name, p_name, r_type)

    def test_property_type_values(self):
        """Test Property Type Values"""
        r_type = "PropertyTypes"
        for r_name, r_values in self.spec.get(r_type).items():
            if r_values.get("Properties") is None:
                self._test_property_type_values(r_values, r_name, "", r_type)
            else:
                for p_name, p_values in r_values.get("Properties", {}).items():
                    self._test_property_type_values(p_values, r_name, p_name, r_type)

    def _test_sub_properties(self, resource_name, v_propertyname, v_propertyvalues):
        property_types = self.spec.get("PropertyTypes").keys()
        if (
            "PrimitiveType" not in v_propertyvalues
            and "PrimitiveItemType" not in v_propertyvalues
        ):
            v_propertytype = ""
            if "ItemType" in v_propertyvalues:
                v_propertytype = v_propertyvalues["ItemType"]
            elif "Type" in v_propertyvalues:
                v_propertytype = v_propertyvalues["Type"]

            v_subproperty_type = str.format("{0}.{1}", resource_name, v_propertytype)

            property_exists = False
            if v_subproperty_type in property_types:
                property_exists = True
            elif v_propertytype in property_types:
                # Special: There is a "Tag" Property type that's used as a "CatchAll" mechanism for Tags,
                # If the subproperty is not found, check if it exists with the resource in the property
                property_exists = True

            self.assertEqual(
                property_exists,
                True,
                "Specified property type {} not found for property {}".format(
                    v_subproperty_type, v_propertyname
                ),
            )

    def test_sub_properties(self):
        """Test Resource sub-Property definitions"""
        # Test properties from resources
        for v_name, v_values in self.spec.get("ResourceTypes").items():
            v_value_properties = v_values.get("Properties", {})
            for p_name, p_values in v_value_properties.items():
                self._test_sub_properties(v_name, p_name, p_values)

        # Test properties from subproperties
        for v_name, v_values in self.spec.get("PropertyTypes").items():
            # Grab the resource part from the subproperty
            resource_name = v_name.split(".", 1)[0]
            if resource_name:
                v_value_properties = v_values.get("Properties", {})
                if v_value_properties is None:
                    self._test_sub_properties(resource_name, "", v_values)
                else:
                    for p_name, p_values in v_value_properties.items():
                        self._test_sub_properties(resource_name, p_name, p_values)

    def test_intrinsic_value_types(self):
        """Test Intrinsic Types"""

        for _, i_value in self.spec.get("IntrinsicTypes").items():
            self.assertIn("Documentation", i_value)
            self.assertIn("ReturnTypes", i_value)
            for return_type in i_value.get("ReturnTypes"):
                self.assertIn(return_type, ["Singular", "List"])

    def test_z_property_value_types(self):
        """Test Property Value Types"""
        for v_name, v_values in self.spec.get("ValueTypes").items():
            self.assertIn(
                v_name,
                self.found_value_types,
                "Value type {} is not used".format(v_name),
            )
            list_count = 0
            number_count = 0
            string_count = 0

            number_max = 0
            number_min = 0
            for p_name, p_values in v_values.items():
                self.assertIn(
                    p_name,
                    [
                        "Ref",
                        "GetAtt",
                        "AllowedValues",
                        "AllowedPattern",
                        "AllowedPatternRegex",
                        "ListMin",
                        "ListMax",
                        "JsonMax",
                        "NumberMax",
                        "NumberMin",
                        "StringMax",
                        "StringMin",
                    ],
                )

                if p_name == "NumberMin":
                    number_min = p_values
                if p_name == "NumberMax":
                    number_max = p_values
                if p_name in ["ListMin", "ListMax"]:
                    list_count += 1
                if p_name in ["NumberMin", "NumberMax"]:
                    number_count += 1
                if p_name in ["StringMin", "StringMax"]:
                    string_count += 1
                if p_name == "Ref":
                    self.assertIsInstance(
                        p_values, dict, "ValueTypes: %s, Type: %s" % (v_name, p_name)
                    )
                    for r_name, r_value in p_values.items():
                        self.assertIn(
                            r_name,
                            ["Resources", "Parameters"],
                            "ValueTypes: %s, Type: %s, Additional Type: %s"
                            % (v_name, p_name, r_name),
                        )
                        self.assertIsInstance(
                            r_value,
                            list,
                            "ValueTypes: %s, Type: %s, Additional Type: %s"
                            % (v_name, p_name, r_name),
                        )
                        if r_name == "Parameters":
                            for r_list_value in r_value:
                                self.assertIsInstance(
                                    r_list_value,
                                    str,
                                    "ValueTypes: %s, Type: %s, Additional Type: %s"
                                    % (v_name, p_name, r_name),
                                )
                                self.assertIn(
                                    r_list_value,
                                    self.spec.get("ParameterTypes"),
                                    "ValueTypes: %s, Type: %s, Additional Type: %s"
                                    % (v_name, p_name, r_name),
                                )
                        elif r_name == "Resources":
                            for r_list_value in r_value:
                                self.assertIsInstance(
                                    r_list_value,
                                    str,
                                    "ValueTypes: %s, Type: %s, Additional Type: %s"
                                    % (v_name, p_name, r_name),
                                )
                                self.assertIn(
                                    r_list_value,
                                    self.spec.get("ResourceTypes"),
                                    "ValueTypes: %s, Type: %s, Additional Type: %s"
                                    % (v_name, p_name, r_name),
                                )

                elif p_name == "GetAtt":
                    self.assertIsInstance(
                        p_values, dict, "ValueTypes: %s, Type: %s" % (v_name, p_name)
                    )
                    for g_name, g_value in p_values.items():
                        self.assertIsInstance(
                            g_value,
                            (str, list),
                            "ValueTypes: %s, Type: %s, Additional Type: %s"
                            % (v_name, p_name, g_name),
                        )
                        self.assertIn(
                            g_name,
                            self.spec.get("ResourceTypes"),
                            "ValueTypes: %s, Type: %s, Additional Type: %s"
                            % (v_name, p_name, g_name),
                        )
                        values = g_value
                        if isinstance(values, str):
                            values = [values]
                        for value in values:
                            self.assertIn(
                                value,
                                self.spec.get("ResourceTypes", {})
                                .get(g_name, {})
                                .get("Attributes", {}),
                                "ValueTypes: %s, Type: %s, Additional Type: %s"
                                % (v_name, p_name, g_name),
                            )
                elif p_name == "AllowedValues":
                    self.assertIsInstance(p_values, list)
                    for l_value in p_values:
                        self.assertIsInstance(
                            l_value, str, "ValueTypes: %s, Type: %s" % (v_name, p_name)
                        )
            self.assertIn(
                list_count, [0, 2], "Both ListMin and ListMax must be specified"
            )
            self.assertIn(
                number_count, [0, 2], "Both NumberMin and NumberMax must be specified"
            )
            self.assertIn(
                string_count, [0, 2], "Both StringMin and StringMax must be specified"
            )
            if number_count == 2:
                self.assertTrue(
                    (number_max > number_min),
                    "NumberMax must be greater than NumberMin",
                )

    def test_parameter_types(self):
        """Test Parameter Types"""
        aws_parameter_types = [
            "AWS::EC2::AvailabilityZone::Name",
            "AWS::EC2::Image::Id",
            "AWS::EC2::Instance::Id",
            "AWS::EC2::KeyPair::KeyName",
            "AWS::EC2::SecurityGroup::GroupName",
            "AWS::EC2::SecurityGroup::Id",
            "AWS::EC2::Subnet::Id",
            "AWS::EC2::Volume::Id",
            "AWS::EC2::VPC::Id",
            "AWS::Route53::HostedZone::Id",
        ]
        valid_types = [
            "String",
            "Number",
            "List<Number>",
            "CommaDelimitedList",
            "List<String>",
            "AWS::SSM::Parameter::Name",
            "AWS::SSM::Parameter::Value<String>",
            "AWS::SSM::Parameter::Value<List<String>>",
            "AWS::SSM::Parameter::Value<CommaDelimitedList>",
        ]
        for aws_parameter_type in aws_parameter_types:
            valid_types.append(aws_parameter_type)
            valid_types.append("List<%s>" % aws_parameter_type)
            valid_types.append("AWS::SSM::Parameter::Value<%s>" % aws_parameter_type)
            valid_types.append(
                "AWS::SSM::Parameter::Value<List<%s>>" % aws_parameter_type
            )

        for v_name, v_values in self.spec.get("ParameterTypes").items():
            self.assertIsInstance(v_values, list, "Parameter Type: %s" % (v_name))



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.resources.backup.BackupPlanLifecycleRule import (
    BackupPlanLifecycleRule,  # pylint: disable=E0401
)


class TestBackupPlanLifecycleRule(BaseRuleTestCase):
    """Check Backup Plan rules with lifecycle has minimum 90 day period between cold and delete"""

    def setUp(self):
        """Setup"""
        super(TestBackupPlanLifecycleRule, self).setUp()
        self.collection.register(BackupPlanLifecycleRule())
        self.success_templates = [
            "test/fixtures/templates/good/resources/backup/test_backup_plan_lifecycle_rule.yml",
        ]

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_negative(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/backup/test_backup_plan_lifecycle_rule.yml",
            1,
        )
